﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Management;
using System.Text.RegularExpressions;

using Microsoft.VisualBasic;
using System.Diagnostics;



namespace DCS_2000E_Wave
{
    public partial class Form_Wave : Form
    {

        public const int mem_length = (int)100e3;
 
        public Form_Wave()
        {
            InitializeComponent();

            //Button clear
            cmbCOMPorts.Enabled = true;
            btnCloseDev.Enabled = false;
            btnRead.Enabled = false;
            btnReadPict.Enabled = false;
            BtnOpenDev.Enabled = true;

            // COM port Listing
            cmbCOMPorts.Items.Clear();

            List<string> portlist = new List<string>(System.IO.Ports.SerialPort.GetPortNames());

//            ManagementObjectCollection moc = new ManagementClass("Win32_PnPDevice").GetInstances(); // Win10 or higher NG
//            ManagementObjectCollection moc = new ManagementClass("Win32_PnpEntry").GetInstances(); // Win10 or higher NG
                ManagementObjectCollection moc = new ManagementClass("Win32_SerialPort").GetInstances();


            var b =
                from string str in portlist
                from ManagementObject mo in moc
                let devName = mo["Name"].ToString()
                let devID = mo["DeviceID"].ToString()
                where Regex.IsMatch(devName, str + "[^0-9]")
                select new { str, devName};

            foreach (var pn in b)
            {
                cmbCOMPorts.Items.Add(pn.str + ": " + pn.devName);
            }

            cmbCOMPorts.SelectedIndex = 0;
        }

        /// <summary>
        /// Open Device port
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BtnOpenDev_Click(object sender, EventArgs e)
        {
            //msg clear
            txtReadBuf.Text = "";
            txtReadWave.Text = "";

            // Port Setting 
            string[] strPortDesc = cmbCOMPorts.Text.Split(':');
            serialPort1.PortName = strPortDesc[0] ;

            serialPort1.BaudRate = 9600;        // USB CDC Class

            serialPort1.DataBits = 8;
            serialPort1.Parity = System.IO.Ports.Parity.None;
            serialPort1.StopBits = System.IO.Ports.StopBits.One;
            serialPort1.Handshake = System.IO.Ports.Handshake.None ;
            serialPort1.ReadTimeout = 3000;
            serialPort1.WriteTimeout = 500;
            serialPort1.ReadBufferSize = 4096;
            serialPort1.NewLine = "\n";
            serialPort1.Open();
            serialPort1.DiscardInBuffer();
            cmbCOMPorts.Enabled = false;

            while (serialPort1.BytesToRead != 0)
            {
                serialPort1.DiscardInBuffer();
                System.Threading.Thread.Sleep(100);
            }

            // IDN query
            serialPort1.WriteLine("*idn?");
            System.Threading.Thread.Sleep(50);
            try
            {
                txtReadBuf.Text = serialPort1.ReadLine();

                //btn setting
                BtnOpenDev.Enabled = false; 
                btnCloseDev.Enabled = true;
                btnRead.Enabled = true;
                btnReadPict.Enabled = true;
            }
            catch (System.Exception caught)
            {
                MessageBox.Show(caught.Message, "Open Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                serialPort1.Close();
                cmbCOMPorts.Enabled = true;
                btnCloseDev.Enabled = false;
                btnRead.Enabled = false;
                btnReadPict.Enabled = false; 
                BtnOpenDev.Enabled = true;
            }
            // MEM size to 10k point
            serialPort1.WriteLine(":acq:reco " + mem_length.ToString());
            System.Threading.Thread.Sleep(50);
        }

        /// <summary>
        /// Close Device Port
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCloseDev_Click(object sender, EventArgs e)
        {

            System.Threading.Thread.Sleep(50);
            serialPort1.Close();
            cmbCOMPorts.Enabled = true;
            btnCloseDev.Enabled = false;
            btnRead.Enabled = false;
            btnReadPict.Enabled = false;
            BtnOpenDev.Enabled = true;
        }

        
        /// <summary>
        /// Read Wave Data
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRead_Click(object sender, EventArgs e)
        {
            string tmpstr = "";
            int requestCnt;
            int iTemp;
            byte[] buf = new byte[mem_length * 2 + 1025];
            int[] sbuf = new int[mem_length + 2];

            while (checkBox2.Checked == false)
            {

                serialPort1.DiscardInBuffer();

                serialPort1.WriteLine(":acq1:mem?");
                System.Threading.Thread.Sleep(50);

                // Wave Header read
                try
                {
                    tmpstr = serialPort1.ReadLine();
                }
                catch (System.Exception caught)
                {
                    MessageBox.Show(caught.Message, "Open Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    tmpstr = "";
                }

                txtReadBuf.Text = tmpstr;
                txtReadWave.Text = "";

                System.Diagnostics.Stopwatch sw = System.Diagnostics.Stopwatch.StartNew();

                btnRead.Enabled = false;
                btnReadPict.Enabled = false;
                btnCloseDev.Enabled = false;


                // Wave data Read
                try
                {

                    for (int index = 0; index < 10; index++)
                    {
                        System.Threading.Thread.Sleep(10);
                        if (serialPort1.BytesToRead > 1000)
                            break;
                    }
                    serialPort1.Read(buf, 0, 2);        // Binary Header
                    buf[2] = 0;
                    iTemp = BitConverter.ToInt16(buf, 1) - 48;	//文字数
                    serialPort1.Read(buf, 0, iTemp);        // Binary count
                    buf[iTemp] = 0;
                    tmpstr = System.Text.Encoding.ASCII.GetString(buf);

                    iTemp = int.Parse(tmpstr.Substring (0 ,iTemp));
                    requestCnt = iTemp;              // Datasize
                    
                    txtReadBuf.Text = txtReadBuf.Text + tmpstr;
                    System.Windows.Forms.Application.DoEvents();

//--------------------------------------------------------------------------
                int iindex = 0;
                int jindex = 0;
                while (iindex  < requestCnt - 1024)
                {
                    if (serialPort1.BytesToRead > 1024)
                    {
                        iindex += serialPort1.Read(buf, iindex, 1024);    //  data
                        jindex = 0;
                    }
                    else if (jindex > 300)
                    {  
                        break;
                    }
                    else 
                    {
                        System.Threading.Thread.Sleep(5);
                        //System.Windows.Forms.Application.DoEvents();
                        jindex++;                        
                    }
                }
                iindex += serialPort1.Read(buf, iindex, 1024);    //  data

//--------------------------------------------------------------------------

                    serialPort1.DiscardInBuffer();
                    serialPort1.DiscardOutBuffer();

                    sw.Stop();
                    txtReadBuf.Text = txtReadBuf.Text + ", " + iindex.ToString() + ", " + sw.Elapsed.ToString();
                    System.Windows.Forms.Application.DoEvents();

                    sw.Start();

                tmpstr = "";
                for (int index = 0; index < mem_length; index++)
                {
                    // Endian convert (byteswap)
                    iTemp = buf[index * 2];
                    buf[index * 2] = buf[index * 2 + 1];
                    buf[index * 2 + 1] = (byte)iTemp;
                    sbuf[index] = BitConverter.ToInt16(buf, index * 2);
                    //tmpstr = tmpstr + "\r\n" + sbuf[index];
                    System.Windows.Forms.Application.DoEvents();
                }
                txtReadWave.Text = tmpstr + "\r\n";

                //Create object BMP,Graphics 
                Bitmap canvas = new Bitmap(pictureBox1.Width, pictureBox1.Height);
                Graphics g = Graphics.FromImage(canvas);
                g.DrawLine(Pens.Orange, 0, 0, 0, 130);
                g.DrawLine(Pens.Orange, 0, 0, 250, 0);
                g.DrawLine(Pens.Orange, 250, 130, 250, 0);
                g.DrawLine(Pens.Orange, 250, 130, 0, 130);
                g.Dispose();

                //Plot to image
                Color c = Color.OrangeRed;
                int lim;

                for (int index = 0; index < mem_length; index++)
                {
                    lim = (130 - sbuf[index]) / 2;
                    if (lim < 0) lim = 0;
                    if (lim > 130) lim = 130;
                    canvas.SetPixel(index / (mem_length / 250), lim, c);
                    System.Windows.Forms.Application.DoEvents();
                }
                pictureBox1.Image = canvas;
                sw.Stop();
                txtReadBuf.Text = txtReadBuf.Text + ", " + sw.Elapsed.ToString();
                btnRead.Enabled = true;
                btnReadPict.Enabled = true;
                btnCloseDev.Enabled = true;

                if (checkBox1.Checked == true)
                {
                    SaveFileDialog saveFileDialog1 = new SaveFileDialog();
                    saveFileDialog1.Title = "Save Wave data to File";
                    saveFileDialog1.Filter = "TEXT File|*.txt|All File|*.*";
                    saveFileDialog1.OverwritePrompt = true;
                    if (saveFileDialog1.ShowDialog() == DialogResult.OK)
                    {
                        System.IO.StreamWriter writer = new System.IO.StreamWriter(saveFileDialog1.FileName, false, Encoding.GetEncoding("Shift_JIS"));
                        writer.Write(txtReadBuf.Text);
                        writer.Write(txtReadWave.Text);
                        writer.Close();
                    }
                    saveFileDialog1.Dispose();
                }
            }
                catch (System.Exception caught)
                {
                    MessageBox.Show(caught.Message, "Open Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                   
                }            
            }
            
        }

        /// <summary>
        /// Read Pict data 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnReadPict_Click(object sender, EventArgs e)
        {
            string tmpstr = "";
            int requestCnt;

            byte[] buf = new byte[110];

            while (checkBox2.Checked == false)
            {

                System.Diagnostics.Stopwatch sw = System.Diagnostics.Stopwatch.StartNew();

                serialPort1.DiscardInBuffer();
                serialPort1.WriteLine("disp:outp?");
                System.Windows.Forms.Application.DoEvents();
                System.Threading.Thread.Sleep(100);
                btnRead.Enabled = false;
                btnReadPict.Enabled = false;

                try
                {
                    for (int index = 0; index < 10; index++)
                    {
                        System.Threading.Thread.Sleep(100);
                        if (serialPort1.BytesToRead > 100)
                            break;
                    }
                    serialPort1.Read(buf, 0, 2);            // Binary Header
                    buf[2] = 0;
                    requestCnt = BitConverter.ToInt16(buf, 1) - 48;
                    serialPort1.Read(buf, 0, requestCnt);        // Binary Header
                    buf[requestCnt] = 0;
                    tmpstr = System.Text.Encoding.ASCII.GetString(buf);

                    requestCnt = int.Parse(tmpstr);              // Datasize


                    txtReadBuf.Text = "#" + tmpstr + "\r\n";
                    System.Windows.Forms.Application.DoEvents();

                    Byte[] DataBuff = new Byte[requestCnt + 2];

                    int iCount = 0;
                    int iIndex = 0;
                    while (iCount < requestCnt - 1024)
                    {
                        if (serialPort1.BytesToRead > 1024)
                        {
                            serialPort1.Read(DataBuff, iCount, 1024);    // Picture data
                            iCount = iCount + 1024;
                        }
                        else
                            System.Threading.Thread.Sleep(5);
                    }
                    iIndex = serialPort1.BytesToRead;
                    serialPort1.Read(DataBuff, iCount, serialPort1.BytesToRead);    // Picture data last
                    iCount = iCount + iIndex;
                    sw.Stop();
                    this.txtReadBuf.Text += " " + sw.Elapsed.ToString() + "\r\n";


                    //Create object BMP,Graphics 
                    Bitmap canvas = new Bitmap(800, 480);

                    //Plot to image
                    Color c;
                    int iXpoint = 0;
                    int iYpoint = 0;
                    int iStep = 0;
                    int iBlue, iRed, iGreen, iColor;


                    for (int index = 0; index < requestCnt; index += 4)
                    {
                        iStep = DataBuff[index] + (DataBuff[index + 1] * 256);
                        iColor = DataBuff[index + 2] + (DataBuff[index + 3] * 256);
                        iBlue = ((iColor & 0x1F) + 1) * 8 - 1;

                        iGreen = (((iColor >> 5) & 0x3f) + 1) * 4 - 1;
                        iRed = (((iColor >> 11) & 0x1f) + 1) * 8 - 1;
                        c = Color.FromArgb(iRed, iGreen, iBlue);
                        for (iCount = 0; iCount < iStep; iCount++)
                        {
                            canvas.SetPixel(iXpoint, iYpoint, c);
                            iXpoint++;
                            if (iXpoint >= 800)
                            {
                                iXpoint = 0;
                                iYpoint++;
                            }
                        }
                    }


                    btnRead.Enabled = true;
                    btnReadPict.Enabled = true;

                    pictureBox1.Image = canvas;
                    if (checkBox1.Checked == true)
                    {
                        SaveFileDialog saveFileDialog1 = new SaveFileDialog();
                        saveFileDialog1.Title = "Save Image to File";
                        saveFileDialog1.Filter = "PNG File|*.png|All File|*.*";
                        saveFileDialog1.OverwritePrompt = true;
                        if (saveFileDialog1.ShowDialog() == DialogResult.OK)
                        {
                            canvas.Save(saveFileDialog1.FileName, System.Drawing.Imaging.ImageFormat.Png);
                        }
                        saveFileDialog1.Dispose();
                    }

                }
                catch (System.Exception caught)
                {
                    MessageBox.Show(caught.Message, "Open Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
    }
}
